*+
*+
*+    Source Module => WHILETST.PRG
*+
*+    Functions: Function AST()
*+
*+    Reformatted by Click! 1.13b3 on May-25-1998 at  2:06 pm
*+
*+

#include "reader.ch"

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Program.......WhileTest.PRG
// Purpose.......Demonstration of modifications made to Clipper GET system to
//               add in a "WHILE <udf>" clause.  This demo just puts a clock
//               in the upper right hand part of screen, but you could use the
//               WHILE udf to do virtually anything.
//
// Author........Mark Cooperstein
//
// Caveats......I use a public variable called nNextPollTime to set the
//              frequency that the WHILE udf is serviced.  In this example
//              its set to once/second.  If you eliminate this feature in your
//              WHILE udf or make it too small, you will find that the READ
//              becomes choppy as the system is spending as much time servicing
//              the WHILE udf as it is the READ.  Generally, a second should
//              be more then adequate for most background processing.  Also,
//              you want your background udf to complete quickly.  If it doesnt
//              it will also tend to slow down or "hang" the READ in progress.
//
// Compile instruction:
//  CLIPPER WHILETST /v /w
//  CLIPPER MULTITSK /v /w
//
// Link instruction:
//  BLINKER @WHILETST.LNK  or
//  RTLINK @WHILETST.LNK
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

local cName
local cAddr1
local cAddr2
local cCity
local cState
local GetList := {}

memvar nNextPollTime                    // so we can compile with the /w option

public nNextPollTime

// initialize some stuff
nNextPollTime := seconds() + 1
cName         := cAddr1 := cAddr2 := space( 30 )
cCity         := space( 20 )
cState        := ' '

cls
@  5, 10 say 'Enter in Name: '  get cName while AST()         
@  6, 10 say 'Enter in Addr1:'  get cAddr1 while AST()        
@  7, 10 say 'Enter in Addr2:'  get cAddr2 while AST()        
@  8, 10 say 'Enter in City: '  get cCity while AST()         
@  9, 10 say 'Enter in State: ' get cState while AST()        

read

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Function......AST
// Purpose.......Provides an Asynchronous Serial Trap used by the WHILE
//                   clause in the modified "@ ... GET <var> ... WHILE <udf>"
//                   command found in modified "reader.ch". I
//      have the frequency of AST set to once/second so
//                   that it doesn't "hog" the Clipper READ in progress.
//               This example udf does nothing but display a continuously
//               updated clock on the top of the screen.
//
// Author........Mark Cooperstein
//     Copyright 1993 Tennessee Eastman Division
// Date..........August 15, 1992
// Parameters....None
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

*+
*+
*+    Function AST()
*+
*+    Called from ( whiletst.prg )   5 - static function keypoller()
*+
*+
*+
function AST( oBrowse )

local cTime          := time()
local cHrs
memvar nNextPollTime

if seconds() < nNextPollTime
   return NIL
endif

cHrs := str( int( val( substr( cTime, 1, 2 ) ) % 12 ), 2 )
cHrs := if( cHrs = ' 0', '12', cHrs )

@  0, 68 say cHrs + substr( cTime, 3 ) + if( val( substr( cTime, 1, 2 ) ) >= 12, ;         
        ' pm', ' am' ) Color 'w/n'

// set time that this udf will be serviced next
nNextPollTime := seconds() + 1          // Only poll once per second.

return NIL

*+ EOF: WHILETST.PRG
